<?php

namespace Bydedi\EditorCms;

use Bydedi\Database\ConnectionInterface;
use Bydedi\Database\Query;
use Bydedi\I18nEditor\Exceptions\ValidationException;
use Bydedi\I18nEditor\Forms\Form;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\Templating\EngineInterface;

class EditorCms implements EditorCmsInterface
{
    protected $i18n_prefix = 'cms_form_';
    /**
     * @var EngineInterface
     */
    protected $templateEngine;

    /**
     * @var ConnectionInterface
     */
    protected $connection;

    /**
     * @var array
     */
    protected $settings;

    /**
     * @var RequestStack
     */
    protected $requestStack;

    /**
     * @var string
     */
    protected $i18nPostfix = '_i18n';

    /**
     * @var array
     */
    protected $languages;

    public function __construct(EngineInterface $templateEngine, ConnectionInterface $connection, array $settings, \Bydedi\Utils\I18n\Text $text)
    {
        $this->templateEngine = $templateEngine;
        $this->connection = $connection;
        $this->settings = $settings;
        $this->text = $text;
    }

    /**
     * @param RequestStack $requestStack
     */
    public function setRequestStack(RequestStack $requestStack)
    {
        $this->requestStack = $requestStack;
    }

    /**
     * @param array $languages
     */
    public function setLanguages(array $languages)
    {
        $this->languages = $languages;
    }

    public function index($editRoute, $listRoute, $createRoute)
    {
        $session = $this->requestStack->getMasterRequest()->getSession();

        if ($this->getRequest()->get('action_2')) {
            $session->set('filterString', '');
            $session->set('selectedLanguage', 'fr');
        }

        if ($this->getRequest()->get('action_1')) {
            if ($this->getRequest()->get('language-select')) {
                $selectedLanguage = $this->getRequest()->get('language-select');
                $session->set('selectedLanguage', $selectedLanguage);
            }

            if ($this->getRequest()->get('filterString')) {
                $filterString = $this->getRequest()->get('filterString');
                $session->set('filterString', $filterString);
            }
        }

        $selectedLanguage = $session->get('selectedLanguage', 'fr');
        $filterString = $session->get('filterString');

        $translateOriginalCodes = $this->getAllTranslatable($filterString);
        $ids = $this->extractCodeIds($translateOriginalCodes);
        $itemsData = $this->getAllItems($this->getRequest()->get('i18n_lang', $selectedLanguage), $ids);
        $items = [];
        foreach ($itemsData as $item) {
            $items[$item[$this->settings['table_name']."_id"]] = [
                'id' => $item['id']
            ];
            foreach ($this->settings['text_columns'] as $column)
            {
                $items[$item[$this->settings['table_name']."_id"]][$column] = $item[$column];
            }
            foreach ($this->settings['textarea_columns'] as $column)
            {
                $items[$item[$this->settings['table_name']."_id"]][$column] = $item[$column];
            }
        }

        $languages = $this->languages;
        $settings = $this->settings;

        return $this->templateEngine->render('cms_list.tpl', compact('items', 'editRoute', 'createRoute', 'languages', 'listRoute', 'selectedLanguage', 'settings', 'translateOriginalCodes', 'filterString'));
    }

    public function edit($editPostRoute, $errors, $listRoute)
    {
        $session = $this->requestStack->getMasterRequest()->getSession();
        $id = $this->getRequest()->get('id');
        $item = $this->getItem($id);
        $textFields = $this->settings['text_columns'];
        $textareaFields = $this->settings['textarea_columns'];
        $original_field = $this->getCode($item[$this->settings['table_name'].'_id']);
        $original_field = $original_field[$this->settings['text_code_column_name']];
        $currentLanguage = $this->languages[$session->get('selectedLanguage', 'fr')];
        $currentLanguageCode = $session->get('selectedLanguage', 'fr');
        $i18n_prefix = $this->i18n_prefix;

        return $this->templateEngine->render('cms_edit.tpl', compact('item', 'editPostRoute', 'textFields', 'textareaFields', 'errors', 'original_field', 'languages', 'listRoute', 'currentLanguage', 'currentLanguageCode', 'i18n_prefix'));
    }

    public function update()
    {
        $data['id'] = $this->getRequest()->get($this->i18n_prefix . 'id');
        $columnsForValidate = [];
        foreach ($this->settings['text_columns'] as $column) {
            $data[$column] = $this->getRequest()->get($this->i18n_prefix . $column);
            $columnsForValidate[] = $column;
        }
        foreach ($this->settings['textarea_columns'] as $column) {
            $data[$column] = $this->getRequest()->get($this->i18n_prefix . $column);
            $columnsForValidate[] = $column;
        }

        $message = $this->text->trans('Le champ est obligatoire');
        $textForm = new Form($columnsForValidate, $this->i18n_prefix, $message);

        if ($errors = $textForm->handle($this->getRequest())) {
            throw new ValidationException($errors);
        }

        $this->updateItem($data);
    }

    public function create($createPostRoute, $errors, $listRoute)
    {
        $session = $this->requestStack->getMasterRequest()->getSession();
        $id = $this->getRequest()->get('id');
        $item = $this->getCode($id);
        $textFields = $this->settings['text_columns'];
        $textareaFields = $this->settings['textarea_columns'];
        $original_field = $this->settings['text_code_column_name'];
        $currentLanguage = $this->languages[$session->get('selectedLanguage', 'fr')];
        $currentLanguageCode = $session->get('selectedLanguage', 'fr');
        $i18n_prefix = $this->i18n_prefix;

        return $this->templateEngine->render('cms_create.tpl', compact('item', 'editPostRoute', 'textFields', 'textareaFields', 'errors', 'original_field', 'createPostRoute', 'listRoute', 'currentLanguage', 'currentLanguageCode', 'i18n_prefix'));
    }

    public function createPost()
    {
        $data['id'] = $this->getRequest()->get($this->i18n_prefix . 'id');
        $columnsForValidate = [];
        foreach ($this->settings['text_columns'] as $column) {
            $data[$column] = $this->getRequest()->get($this->i18n_prefix . $column);
            $columnsForValidate[] = $column;
        }
        foreach ($this->settings['textarea_columns'] as $column) {
            $data[$column] = $this->getRequest()->get($this->i18n_prefix . $column);
            $columnsForValidate[] = $column;
        }

        $message = $this->text->trans('Le champ est obligatoire');
        $textForm = new Form($columnsForValidate, $this->i18n_prefix, $message);

        if ($errors = $textForm->handle($this->getRequest())) {
            throw new ValidationException($errors);
        }

        $data['lang'] = $this->getRequest()->getSession()->get('selectedLanguage');

        $this->createItem($data);
    }

    protected function getAllTranslatable($filterString)
    {
        $query = new Query();

        $query
            ->select()
            ->from($this->settings['table_name']);
        if ($filterString != '') {
            $query
                ->where($this->settings['text_code_column_name'] . " LIKE '%" .$filterString . "%'");
        }

        return $this->connection->getAll($query->build());
    }

    protected function getAllItems($lang, $ids)
    {
        $query = new Query();

        $columnsArray = $this->settings['text_columns'];
        $lastKey = array_search(end($columnsArray), $columnsArray);
        $valuesColumns = ', ';
        foreach ($this->settings['text_columns'] as $key => $column) {
            $valuesColumns .= $column . ", ";
        }

        foreach ($this->settings['textarea_columns'] as $key => $column) {
            if ($key == $lastKey) {
                $valuesColumns .= $column;
            } else {
                $valuesColumns .= $column . ", ";
            }
        }

        $query
            ->select('id, '.$this->settings['table_name'].'_id ' . $valuesColumns)
            ->from($this->makeI18nTableName())
            ->where($this->settings['lang_column'], $lang)
            ->where($this->settings['table_name']."_id IN (" . join(",", $ids) . ")")
        ;

        return $this->connection->getAll($query->build());
    }

    protected function makeI18nTableName()
    {
        return $this->settings['table_name'] . $this->i18nPostfix;
    }

    /**
     * @return null|\Symfony\Component\HttpFoundation\Request
     */
    protected function getRequest()
    {
        return $this->requestStack->getCurrentRequest();
    }

    protected function getItem($id)
    {
        $query = new Query();

        $query
            ->select()
            ->from($this->makeI18nTableName())
            ->where('id', $id)
        ;

        return $this->connection->getOne($query->build());
    }

    protected function getCode($id)
    {
        $query = new Query();

        $query
            ->select()
            ->from($this->settings['table_name'])
            ->where('id', $id)
        ;

        return $this->connection->getOne($query->build());
    }

    public function getId()
    {
        return $this->getRequest()->get($this->i18n_prefix . 'id');
    }

    protected function updateItem($data)
    {
        $query = new Query();

        $query
            ->update($this->makeI18nTableName())
        ;

        foreach ($this->settings['text_columns'] as $column) {
            $query
                ->value($column, $data[$column]);
        }
        foreach ($this->settings['textarea_columns'] as $column) {
            $query
                ->value($column, $data[$column]);
        }

        $query
            ->where('id', $data['id'])
        ;

        return $this->connection->query($query->build());
    }

    protected function createItem($data)
    {
        $query = new Query();

        $query
            ->insert($this->makeI18nTableName())
        ;

        foreach ($this->settings['text_columns'] as $column) {
            $query
                ->value($column, $data[$column]);
        }
        foreach ($this->settings['textarea_columns'] as $column) {
            $query
                ->value($column, $data[$column]);
        }

        $query
            ->value($this->settings['table_name'].'_id', $data['id'])
            ->value($this->settings['lang_column'], $data['lang'])
        ;
        return $this->connection->query($query->build());
    }

    protected function extractCodeIds($translateOriginalCodes)
    {
        $ids = [];
        foreach ($translateOriginalCodes as $transData) {
            $ids[] = "'" . $transData['id'] . "'";
        }
        return $ids;
    }
}
